<?php
/*
Plugin Name: .Mac Sidebar
Version: 0.1
Plugin URI: TODO
Description: Retrieves and displays link from a .Mac account on your sidebar. You need to add one line of code to your sidebar.php file.
Author: Marcos Kirsch <dotMacSidebarPlugin@Kirsch.com.mx>
Author URI: http://Marcos.Kirsch.com.mx/
*/


////////////////////////////////////////////////////////////////////////////////////////////
// Version history:
//
// Version 0.1 (March 21, 2006)
// - Renamed to .Mac Sidebar, converted to actual WordPress plugin.
// - Reset version number to 0.1
//
// Version 1.3 (July 25, 2005):
// - Now works with the new layout used in .mac. Configuration is even easier too.
//
// Version 1.2.1 (July 18, 2005):
// - Apple graciously changed the way they generate the sidebar and broke this script.
//   Modified so that it keeps going (instead of dying which is a tad extreme)
//
// Version 1.2 (May 21, 2005):
// - Changed formatting of sidebar output to mach the Kubrick theme for Wordpress
//   but I should really add template-based formatting.
//
// Version 1.1 (May 15, 2005):
// - Using CURL to retrieve file, so that script runs in servers in which
//   URL file-access is disabled (done for security reasons).
//
// Version 1.0 (Feb 1, 2004):
// - First release
//
//////////////////////////////////////////////////////////////////////////////////////////

/* -------------------------
What follows are the functions that display things in your sidebar.
Feel free to customize them to your needs
--------------`----------- */

function show_dotMac_links(
	$dotMacUser, // your .Mac user name
	$numberOfLinks, // Max number of links to display
	$chronologicalOrder = false, // Display in chron order or reverse chron order.
	$target = '_blank' // target for <A> tag
){
	$dotMacPage = "http://homepage.mac.com/".$dotMacUser."/.nav.homepage.js";

	// Retrieve the file
	$curl_handle = curl_init();
	curl_setopt($curl_handle, CURLOPT_URL, $dotMacPage);
	curl_setopt($curl_handle, CURLOPT_CONNECTTIMEOUT, 3);
	curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1);
	$string = curl_exec($curl_handle);
	curl_close($curl_handle);
	if (empty($string)){
		echo ("Could not connect to .mac! (1)");
		return;
	}

	// The file read contains more than just the links. Discard the stuff before
	// the links:
	$array = explode('\'<A', $string);
	$string = '<A'.$array[1];
	if ($string == ""){
		echo ("Could not generate .mac sidebar! (2)");
		return;
	}
	
	// And then throw away the last part where the relevant text finishes.
	$array = explode('\');', $string);
	$string = $array[0];
	if ($string == ""){
		echo ("Could not generate .mac sidebar! (3)");
		return;
	}
	
	// Fantastic. Let's now put the links in an array... notice they are split using &nbsp;
	$temp = explode('&nbsp;', $string);
	
	// and discard all those ugly pipes (|)
	$i = 0;
	foreach ($temp as $line){
		if ($i % 2 == 0) $linkStrings[] = $line; $i++;
	}
	
	// at this point, each element of the array has one link, all as text. Let's parse
	// it to get the name of the page and the url. One of the links corresponds
	// to the page we retrieved (as specified in $dotMacPage). This one doesn't have
	// the url but we now it because it's in $dotMacPage! (duh). This line starts is
	// surrounded by <b> tags.
	$i = 0;
	foreach($linkStrings as $linkString){
		if (eregi("<b>",$linkString)){
			// this is the page we retrieved!
			$link['url'] = $dotMacPage;
			
			// the name is between :none"> and </a>
			$link['name'] = dotMacSidebarPlugin_inBetweenOf (':none">', '</A>', $linkString);
		}else{
			// Get the url... it's the stuff in the href (href="...">)
			$link['url'] = dotMacSidebarPlugin_inBetweenOf ('href="', '">', $linkString);
			if ($link['url'] === false){
				echo ("Could not generate .mac sidebar! (6) i=$i");
				return;
			}
	
			// get the name of the page... it's the stuff between the '>' and '</A>'
			$link['name'] = dotMacSidebarPlugin_inBetweenOf(">", "</A>", $linkString);
			if ($link['name'] === false){
				echo ("Could not generate .mac sidebar! (7) i=$i link[url]=".$link['url']);
				return;
			}
		}
		// put link in array
		$links[] = $link;
		$i++;
	}

	// Print out the sidebar contents (HTML output)
	if ($chronologicalOrder){
		$begin = 0;
		$end = $numberOfLinks;
		$increment = 1;
	}else{
		$begin = $numberOfLinks;
		$end = 0;
		$increment = -1;
	}
	
	if ($chronologicalOrder){
		for ($i = 0; $i < $numberOfLinks; $i++){
			echo '<li><a href="'.$links[$i]['url'].'"';
			if ($target != ""){
				echo ' target="'.$target.'"';
			}
			echo '>'.$links[$i]['name'].'</a></li>'."\n";
		}
	}else{
		for ($i = 0; $i < $numberOfLinks; $i++){
			echo '<li><a href="'.$links[sizeof($links) - $i - 1]['url'].'"';
			if ($target != ""){
				echo ' target="'.$target.'"';
			}
			echo '>'.$links[sizeof($links) - $i - 1]['name'].'</a></li>'."\n";
		}
	}
	/*
	require_once (ABSPATH . WPINC . '/rss-functions.php');
	$rss = @fetch_rss('http://feeds.technorati.com/cosmos/rss/?url='.trailingslashit(get_option('home')) .'&key=$apikey');
	if ( isset($rss->items) && 0 != count($rss->items) ) {
		?>
				
		<li id="incominglinks"><h2><?php _e('Incoming Links'); ?></h2>
		<ul>
		<?php
		$rss->items = array_slice($rss->items, 0, $limit);
		foreach ($rss->items as $item ) {
		?>
		<li><a href="<?php echo wp_filter_kses($item['link']); ?>"><?php echo wp_specialchars($item['title']); ?></a></li>
		<?php } ?>
		</ul></li>

	<?php
	}
	*/
}

// helper function returns the portion of the string that falls between
// the first occurrence of $a and the first occurence of $b after $a
// returns false if $a or $b are not found in the right order
function dotMacSidebarPlugin_inBetweenOf($a, $b, $string){
	$locationOfA = strpos($string, $a);
	if ($locationOfA === false){
		return false;
	}
	// search for $b after that
	$locationOfB = strpos ($string, $b, $locationOfA+1);
	if ($locationOfB === false){
		return false;
	}

	return substr ($string, $locationOfA + strlen($a), $locationOfB - $locationOfA - strlen($a));

}

?>